@@ -30,7 +30,6 @@ import {
   BackendDataSourceResponse,
   DataSourceWithBackend,
   FetchResponse,
-  frameToMetricFindValue,
   getBackendSrv,
   getTemplateSrv,
   TemplateSrv,
@@ -209,6 +208,7 @@ export default class InfluxDatasource extends DataSourceWithBackend<InfluxQuery,
       if (query.adhocFilters?.length) {
         const adhocFiltersToTags: InfluxQueryTag[] = (query.adhocFilters ?? []).map((af) => {
           const { condition, ...asTag } = af;
+          asTag.value = this.templateSrv.replace(asTag.value ?? '', variables);
           return asTag;
         });
         query.tags = [...(query.tags ?? []), ...adhocFiltersToTags];
@@ -325,11 +325,6 @@ export default class InfluxDatasource extends DataSourceWithBackend<InfluxQuery,
   }
 
   interpolateQueryExpr(value: string | string[] = [], variable: QueryVariableModel, query?: string) {
-    // If there is no query just return the value directly
-    if (!query) {
-      return value;
-    }
-
     if (typeof value === 'string') {
       // Check the value is a number. If not run to escape special characters
       if (!isNaN(parseFloat(value))) {
@@ -356,17 +351,30 @@ export default class InfluxDatasource extends DataSourceWithBackend<InfluxQuery,
     // If the variable is not a multi-value variable
     // we want to see how it's been used. If it is used in a regex expression
     // we escape it. Otherwise, we return it directly.
-    // regex below checks if the variable inside /^...$/ (^ and $ is optional)
+    // The regex below searches for regexes within the query string
+    const regexMatcher = new RegExp(
+      /\/((?![*+?])(?:[^\r\n\[/\\]|\\.|\[(?:[^\r\n\]\\]|\\.)*\])+)\/((?:g(?:im?|mi?)?|i(?:gm?|mg?)?|m(?:gi?|ig?)?)?)/,
+      'gm'
+    );
+    // If matches are found this regex is evaluated to check if the variable is contained in the regex /^...$/ (^ and $ is optional)
     // i.e. /^$myVar$/ or /$myVar/ or /^($myVar)$/
     const regex = new RegExp(`\\/(?:\\^)?(.*)(\\$${variable.name})(.*)(?:\\$)?\\/`, 'gm');
-    if (regex.test(query)) {
-      if (typeof value === 'string') {
-        return escapeRegex(value);
+    if (!query) {
+      return value;
+    }
+
+    const queryMatches = query.match(regexMatcher);
+    if (!queryMatches) {
+      return value;
+    }
+    for (const match of queryMatches) {
+      if (!match.match(regex)) {
+        continue;
       }
 
       // If the value is a string array first escape them then join them with pipe
       // then put inside parenthesis.
-      return `(${value.map((v) => escapeRegex(v)).join('|')})`;
+      return typeof value === 'string' ? escapeRegex(value) : `(${value.map((v) => escapeRegex(v)).join('|')})`;
     }
 
     return value;
@@ -414,13 +422,23 @@ export default class InfluxDatasource extends DataSourceWithBackend<InfluxQuery,
   }
 
   toMetricFindValue(rsp: DataQueryResponse): MetricFindValue[] {
-    const data = rsp.data ?? [];
+    const valueMap = new Map<string, MetricFindValue>();
     // Create MetricFindValue object for all frames
-    const values = data.map((d) => frameToMetricFindValue(d)).flat();
-    // Filter out duplicate elements
-    return values.filter((elm, idx, self) => idx === self.findIndex((t) => t.text === elm.text));
+    rsp?.data?.forEach((frame: DataFrame) => {
+      if (frame && frame.length > 0) {
+        let field = frame.fields.find((f) => f.type === FieldType.string);
+        if (!field) {
+          field = frame.fields.find((f) => f.type !== FieldType.time);
+        }
+        if (field) {
+          field.values.forEach((v) => {
+            valueMap.set(v.toString(), { text: v.toString() });
+          });
+        }
+      }
+    });
+    return Array.from(valueMap.values());
   }
-
   // By implementing getTagKeys and getTagValues we add ad-hoc filters functionality
   // Used in public/app/features/variables/adhoc/picker/AdHocFilterKey.tsx::fetchFilterKeys
   getTagKeys(options?: DataSourceGetTagKeysOptions<InfluxQuery>) {
